'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _isPromise = require('zent-utils/isPromise');

var _isPromise2 = _interopRequireDefault(_isPromise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var handleSubmit = function handleSubmit(submit, zentForm) {
  var props = zentForm.props;
  var values = zentForm.getFormValues();
  var onSubmitSuccess = props.onSubmitSuccess,
      onSubmitFail = props.onSubmitFail;


  zentForm.setFormPristine(false);

  if (zentForm.isValid()) {
    var doSubmit = function doSubmit() {
      var result = void 0;

      try {
        result = submit(values);
      } catch (submitError) {
        if (onSubmitFail) {
          onSubmitFail(submitError);
        }
        return submitError;
      }

      if ((0, _isPromise2['default'])(result)) {
        zentForm.setState({
          isSubmitting: true
        });

        return result.then(function (submitResult) {
          zentForm.setState({
            isSubmitting: false
          });
          if (onSubmitSuccess) {
            onSubmitSuccess(submitResult);
          }
          return submitResult;
        }, function (submitError) {
          zentForm.setState({
            isSubmitting: false
          });
          if (onSubmitFail) {
            onSubmitFail(submitError);
          }
          return submitError;
        });
      }

      // submit是一个同步过程，直接当成功处理
      if (onSubmitSuccess) {
        onSubmitSuccess(result);
      }

      return result;
    };

    return doSubmit();
  }
};

exports['default'] = handleSubmit;
module.exports = exports['default'];